from sqlalchemy import (
    Integer,
    Column,
    String,
    DateTime,
    Text,
    Boolean,
)
from datetime import datetime
from ...database import Base


class NCTStudyModel(Base):
    __tablename__ = "nct_study"

    id = Column(Integer, primary_key=True)
    nct_id = Column(String(128), nullable=False, unique=True)
    brief_title = Column(Text)
    study_status = Column(String(128))
    study_type = Column(String(50))
    study_start_date = Column(DateTime)
    last_update_submitted_qc_date = Column(DateTime)
    phase = Column(Text)
    sponsors = Column(Text)
    conditions = Column(Text)
    industry_flag = Column(Boolean)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from sqlalchemy import (
    Column,
    String,
)

from ...database import Base


class NDCSetIDCrossModel(Base):
    __tablename__ = "ndc_set_id"

    product_ndc = Column(String(50), nullable=True, primary_key=True)
    package_ndc = Column(String(4000), nullable=True)
    spl_set_id = Column(String(4000), nullable=True, index=True)



from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm.exc import NoResultFound
from marshmallow import ValidationError

from ...database import db_session
from ...resources.PatentApplication.models import (
    PatentApplicationModel,
)
from .schemas import (
    PatentApplicationResourceSchema,
    PatentApplicationQueryParamsSchema,
)
from ...utils.utils import update_model


schema_resource = PatentApplicationResourceSchema()
schema_params = PatentApplicationQueryParamsSchema()


class PatentApplication:
    @staticmethod
    def create(params):
        """
        Args:
            params: Dict (PatentApplicationResourceSchema)

        Returns:
            PatentApplicationResourceSchema

        Raises:
            ValidationError
            DBException
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        Args:
            params: PatentApplicationQueryParamsSchema

        Returns:
            List<PatentApplicationResourceSchema>

        Raises:
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        patent_application_query = _build_query(params=data)
        response = schema_resource.dump(
            patent_application_query, many=True).data
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: PatentApplicationQueryParamsSchema

        Returns:
            PatentApplicationResourceSchema

        Raises:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        patent_application_query = _build_query(params=data).one()
        response = schema_resource.dump(patent_application_query).data
        return response

    @staticmethod
    def delete(id):
        """
        :param
            id
        :return:
            delete message
        :exception:
            sqlalchemy.orm.exc.NoResultFound
            sqlalchemy.orm.exc.MultipleResultsFound
            SQLAlchemyError
        """

        patent_application_query = db_session.query(
            PatentApplicationModel).filter_by(
            id=id,
        ).one()
        try:
            db_session.delete(patent_application_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise

    @staticmethod
    def upsert(params):
        """
        Args:
            params: PatentApplicationResourceSchema

        Returns:
            PatentApplicationResourceSchema

        Raises:
            ValidationError
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        try:
            query_params = {
                'application_number': params['application_number'],
                'jurisdiction': params['jurisdiction'],
            }
            patent_application_query = _build_query(query_params).one()
            response = _helper_update(data, patent_application_query)
        except NoResultFound:
            response = _helper_create(data)
        return response


def _helper_create(data):
    new_patent_application = PatentApplicationModel(**data)
    try:
        db_session.add(new_patent_application)
        db_session.commit()
        patent_application_query = db_session.query(
            PatentApplicationModel).get(
            new_patent_application.id)
        response = schema_resource.dump(patent_application_query).data
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_update(data, patent_application_query):
    data['id'] = patent_application_query.id
    data['application_number'] = patent_application_query.application_number
    data['jurisdiction'] = patent_application_query.jurisdiction
    try:
        update_model(data, patent_application_query)
        db_session.commit()
        response = schema_resource.dump(patent_application_query).data
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise


def _build_query(params):
    q = db_session.query(
        PatentApplicationModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('application_number'):
        q = q.filter_by(application_number=params.get('application_number'))
    if params.get('jurisdiction'):
        q = q.filter_by(jurisdiction=params.get('jurisdiction'))
    return q



from marshmallow import (
    Schema,
    fields,
    validate,
)


class DesignationOutboxResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    intervention_condition_id = fields.Integer(allow_none=True)
    designation = fields.String(validate=not_blank, required=True)
    designation_text = fields.String(validate=not_blank)
    news_id = fields.Integer(required=True)
    updated_at = fields.DateTime()



from marshmallow import (
    Schema,
    fields,
)


class ClaimChallengedResourceSchema(Schema):
    id = fields.Integer(dump_only=True)
    ptab2_proceeding_id = fields.Integer(required=True)
    claim_id = fields.Integer(required=True)
    prior_art_combination = fields.Integer(required=True)
    prior_art_id = fields.Integer(required=True)
    prior_art_nature = fields.String(required=True, allow_none=True)
    nature_of_challenge = fields.String(required=True, allow_none=True)
    updated_at = fields.DateTime()


class ClaimChallengedQueryParamsSchema(Schema):
    id = fields.Integer()
    ptab2_proceeding_id = fields.Integer()
    claim_id = fields.Integer()
    prior_art_combination = fields.Integer()
    prior_art_id = fields.Integer()
    prior_art_nature = fields.String()




from src.cs_models.aact_database import db_session
from src.cs_models.resources_aact.Study.models import StudyModel
from src.cs_models.resources_aact.Sponsor.models import SponsorModel
from src.cs_models.resources_aact.Intervention.models import InterventionModel
from src.cs_models.resources_aact.Condition.models import ConditionModel
from src.cs_models.resources_aact.StudyReference.models import (
    StudyReferenceModel,
)
from sqlalchemy.orm import aliased

study = aliased(StudyModel)
sponsor = aliased(SponsorModel)
intervention = aliased(InterventionModel)
study_ref = aliased(StudyReferenceModel)
condition = aliased(ConditionModel)
# q = db_session.query(
#     study).filter_by(nct_id='NCT03626311')

# q = db_session.query(
#         study.nct_id,
#         study.phase,
#         sponsor.name,
#         intervention.name,
#         study.official_title,
# ).join(sponsor, study.nct_id == sponsor.nct_id
# ).join(intervention, study.nct_id == intervention.nct_id
# ).filter(
#     intervention.name.op('~')('MRTX849')
# )

q = db_session.query(
    study.nct_id,
    study.phase,
    intervention.name,
    study_ref.pmid,
    sponsor.name,
    condition.name,
).join(
    sponsor,
    study.nct_id == sponsor.nct_id,
).outerjoin(
    intervention,
    study.nct_id == intervention.nct_id,
).outerjoin(
    study_ref,
    study.nct_id == study_ref.nct_id,
).outerjoin(
    condition,
    study.nct_id == condition.nct_id
).filter(
    intervention.name.op('~')('MRTX849')
    # study.nct_id == 'NCT04141787'
)

for rec in q.all():
    print(rec)



from sqlalchemy import (
    Column,
    Integer,
    String,
    Text,
    DateTime,
    UniqueConstraint)
from datetime import datetime

from ...database import Base


class TrialStatModel(Base):
    __tablename__ = 'trial_stats'

    id = Column(Integer, primary_key=True)

    stat_type = Column(String(190), nullable=False)
    name = Column(String(190), nullable=False)
    ref_counts_by_type = Column(Text)
    id_count = Column(Integer)
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )

    __table_args__ = (
        UniqueConstraint('stat_type', 'name'),
    )



import os
from sqlalchemy import create_engine
from sqlalchemy.orm import scoped_session, sessionmaker
from sqlalchemy.ext.declarative import declarative_base

postgres_db_uri = "postgres+psycopg2://{user}:{password}@{host}:{port}/{db}".format(
    user=os.getenv("AACT_USER", "root"),
    password=os.getenv("AACT_PASSWORD", "testpass"),
    host=os.getenv("AACT_HOST", "127.0.0.1"),
    port=5432,
    db=os.getenv("AACT_DB", "aact"),
)
engine = create_engine(postgres_db_uri, convert_unicode=True)

db_session = scoped_session(
    sessionmaker(autocommit=False, autoflush=False, bind=engine)
)

Base = declarative_base()
Base.query = db_session.query_property()



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
    ValidationError,
)
from ...utils.utils import pre_load_date_fields


class InterventionMilestoneOutboxResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    date = fields.DateTime(required=True)
    news_id = fields.Integer(allow_none=True)
    sec_filing_id = fields.Integer(allow_none=True)
    note = fields.String(validate=not_blank, required=True)
    intervention_id = fields.Integer(allow_none=True)
    condition_id = fields.Integer(allow_none=True)
    start_date = fields.DateTime(allow_none=True)
    end_date = fields.DateTime(allow_none=True)
    updated_at = fields.DateTime()

    @pre_load
    def convert_string_to_datetime(self, in_data):
        date_fields = ['date', 'start_date', 'end_date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format="%Y%m%dT%H%M%S",
        )
        return in_data

    @pre_load
    def check_source_ids(self, in_data):
        if self._get_number_of_source_fields(in_data) != 1:
            raise ValidationError('Provide either news_id or '
                                  'sec_id, not both')

    def _get_number_of_source_fields(self, in_data):
        result = 0
        if 'news_id' in in_data:
            if in_data['news_id'] is not None:
                result += 1
        if 'sec_filing_id' in in_data:
            if in_data['sec_filing_id'] is not None:
                result += 1
        return result




from sqlalchemy import (
    Column,
    Integer,
    String,
    Text,
    DateTime,
)
from datetime import datetime
from ...database import Base


class PubmedModel(Base):
    __tablename__ = 'pubmed'

    id = Column(Integer, primary_key=True)
    pubmed_id = Column(Integer, unique=True)
    doi = Column(String(128), nullable=True)
    publication_type = Column(Text, nullable=True)
    date_created = Column(DateTime, nullable=True)
    date_completed = Column(DateTime, nullable=True)
    date_revised = Column(DateTime, nullable=True)
    journal_issn = Column(String(50), nullable=True)
    journal_volume = Column(String(50), nullable=True)
    journal_issue = Column(String(50), nullable=True)
    journal_title = Column(String(255), nullable=True)
    journal_title_iso_abbrev = Column(String(255), nullable=True)
    title = Column(Text, nullable=False)
    abstract = Column(Text, nullable=True)
    author_list = Column(Text, nullable=True)
    keyword_list = Column(Text, nullable=True)
    chemical_list = Column(Text, nullable=True)
    mesh_list = Column(Text, nullable=True)
    coi_statement = Column(Text, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



import copy
from marshmallow import ValidationError
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import (
    NoResultFound,
    MultipleResultsFound)
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.OrangeBookExclusivity import (
    OrangeBookExclusivity,
)
from src.cs_models.utils.utils import convert_string_to_datetime


class OrangeBookExclusivityResourceTestCase(TestCase):
    def setUp(self):
        super(OrangeBookExclusivityResourceTestCase, self).setUp()
        self.inst = OrangeBookExclusivity()

        self.ob_exclusivity1 = self.inst.create({
            'appl_no': '1000',
            'appl_type': 'A',
            'product_no': '1',
            'exclusivity_code': 'ODE-64',
            'exclusivity_date': 'Apr 4, 2021',
        })

        self.ob_exclusivity2 = self.inst.create({
            'appl_no': '1000',
            'appl_type': 'A',
            'product_no': '2',
            'exclusivity_code': 'ODE-64',
            'exclusivity_date': 'Apr 4, 2021',
        })

        self.ob_exclusivity3 = self.inst.create({
            'appl_no': '2000',
            'appl_type': 'A',
            'product_no': '1',
            'exclusivity_code': 'ODE-64',
            'exclusivity_date': 'Apr 4, 2021',
        })

        self.valid_data = {
            'appl_no': '2000',
            'appl_type': 'A',
            'product_no': '2',
            'exclusivity_code': 'ODE-64',
            'exclusivity_date': 'Apr 5, 2021',
        }

    @parameterized.expand([
        ('appl_type',),
        ('appl_no',),
        ('product_no',),
        ('exclusivity_code',),
    ])
    def test_create_validation_error_missing_field(self, field_to_pop):
        base_data = copy.copy(self.valid_data)
        base_data.pop(field_to_pop)
        self.assertRaises(
            ValidationError,
            self.inst.create,
            base_data,
        )

    def test_create_violates_unique_constraint(self):
        self.assertRaises(
            IntegrityError,
            self.inst.create,
            {
                **self.valid_data,
                **{'product_no': self.ob_exclusivity3['product_no']},
            },
        )

    def test_create(self):
        resp = self.inst.create(self.valid_data)
        expected_data = {
            **self.valid_data,
            **{'exclusivity_date': (
                f"{self.valid_data['exclusivity_date']}+00:00")},
        }
        second_equals_first(
            expected_data,
            resp,
        )

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(3, len(resp))

    @parameterized.expand([
        ('id', 'ob_exclusivity1', 'id', 1),
    ])
    def test_read_w_params(
        self,
        field_name,
        attr,
        attr_field,
        expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(3, len(resp))

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    @parameterized.expand([
        ('id', 999, NoResultFound),
        ('appl_no', '1000', MultipleResultsFound),
    ])
    def test_one_raises_exception(self, field_name, field_value, exception):
        self.assertRaises(
            exception,
            self.inst.one,
            {
                field_name: field_value,
            },
        )

    @parameterized.expand([
        (['id'],),
        (['appl_type', 'appl_no', 'product_no', 'exclusivity_code'],)
    ])
    def test_one(self, query_fields):
        query_params = {}
        for field in query_fields:
            query_params[field] = self.ob_exclusivity1[field]

        resp = self.inst.one(query_params)
        second_equals_first(
            self.ob_exclusivity1,
            resp,
        )

    def test_upsert_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.upsert,
            {
                'appl_no': self.valid_data['appl_no'],
            }
        )

    def test_upsert_creates_new_entry(self):
        data = copy.copy(self.valid_data)
        self.assertEqual(3, len(self.inst.read({})))
        self.inst.upsert(data)
        self.assertEqual(4, len(self.inst.read({})))

    def test_upsert_updates_existing_row(self):
        data = {
            **self.valid_data,
            **{
                'appl_type': self.ob_exclusivity1['appl_type'],
                'appl_no': self.ob_exclusivity1['appl_no'],
                'product_no': self.ob_exclusivity1['product_no'],
                'exclusivity_code': self.ob_exclusivity1['exclusivity_code'],
            },
        }
        self.assertEqual(3, len(self.inst.read({})))
        resp = self.inst.upsert(data)

        second_equals_first(
            {
                **data,
                **{'exclusivity_date': (
                    f"{data['exclusivity_date']}+00:00")}
            },
            resp,
        )
        self.assertEqual(3, len(self.inst.read({})))

    def test_upsert_updates_exclusivity_date(self):
        # greater than what's already there
        raw_exclusivity_date = 'Apr 10, 2021'
        data = {
            **self.valid_data,
            **{
                'appl_type': self.ob_exclusivity1['appl_type'],
                'appl_no': self.ob_exclusivity1['appl_no'],
                'product_no': self.ob_exclusivity1['product_no'],
                'exclusivity_code': self.ob_exclusivity1['exclusivity_code'],
                'exclusivity_date': raw_exclusivity_date,
            },
        }
        self.assertEqual(3, len(self.inst.read({})))
        resp = self.inst.upsert(data)

        exclusivity_date = convert_string_to_datetime(
            date=raw_exclusivity_date,
            string_format='%b %d, %Y',
        )
        expected_exclusivity_date = f"{exclusivity_date}+00:00"

        second_equals_first(
            {
                **data,
                **{'exclusivity_date': expected_exclusivity_date},
            },
            resp,
        )
        self.assertEqual(3, len(self.inst.read({})))

    def test_upsert_does_not_update_exclusivity_date(self):
        # less than what's already there
        raw_exclusivity_date = 'Apr 1, 2021'
        data = {
            **self.valid_data,
            **{
                'appl_type': self.ob_exclusivity1['appl_type'],
                'appl_no': self.ob_exclusivity1['appl_no'],
                'product_no': self.ob_exclusivity1['product_no'],
                'exclusivity_code': self.ob_exclusivity1['exclusivity_code'],
                'exclusivity_date': raw_exclusivity_date,
            },
        }
        self.assertEqual(3, len(self.inst.read({})))
        resp = self.inst.upsert(data)

        expected_exclusivity_date = self.ob_exclusivity1['exclusivity_date']
        second_equals_first(
            {
                **data,
                **{'exclusivity_date': expected_exclusivity_date},
            },
            resp,
        )
        self.assertEqual(3, len(self.inst.read({})))



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
    ValidationError,
)


class ClaimResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    patent_id = fields.Integer()
    patent_application_id = fields.Integer()
    claim_number = fields.Integer(required=True)
    claim_text = fields.String(required=True, validate=not_blank)
    updated_at = fields.DateTime(dump_only=True)

    @pre_load
    def pre_load(self, in_data):
        if 'patent_application_id' in in_data and 'patent_id' in in_data:
            raise ValidationError(
                'Cannot provide both '
                '`patent_application_id` and `patent_id` fields'
            )

        if (
            'patent_application_id' not in in_data and
            'patent_id' not in in_data
        ):
            raise ValidationError(
                'Should provide either '
                '`patent_application_id` or `patent_id` field'
            )


class ClaimQueryParamsSchema(Schema):
    id = fields.Integer()
    patent_id = fields.Integer()
    patent_application_id = fields.Integer()
    claim_number = fields.Integer()


class ClaimPatchSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    claim_text = fields.String(validate=not_blank)



import json
import pytz
from datetime import datetime
from marshmallow import ValidationError
from sqlalchemy.sql.expression import ClauseElement


# converts sql row to dict
def build_object(row):
    d = {}
    if row is None:
        return d

    for column in row.__table__.columns:
        # TODO: Need try-except for python2.7 inside docker container
        # After figuring out venv inside docker, relative imports
        # remove this hack
        try:
            a = str(getattr(row, column.name))
        except UnicodeEncodeError:
            a = getattr(row, column.name).encode('utf-8')
        d[column.name] = a

    return d


# converts sql rows to list of dicts
def build_objects(rows):
    return [build_object(row) for row in rows]


# helper for PATCH
def update_model(resource, model):
    for k, v in resource.items():
        model.__setattr__(k, v)


def get_or_create(session, model, defaults=None, **kwargs):
    instance = session.query(model).filter_by(**kwargs).first()
    if instance:
        return instance, False
    else:
        params = dict((k, v) for k, v in kwargs.items() if
                      not isinstance(v, ClauseElement))
        params.update(defaults or {})
        instance = model(**params)
        session.add(instance)
        return instance, True


# converts returned data to json; used in tests
def to_json(data):
    return json.loads(data.data.decode('utf8'))


def validate_data(schema, params):
    errors = schema.validate(data=params)
    if errors:
        raise ValidationError(errors)


def convert_to_boolean(value):
    """
    >>> convert_to_boolean(True)
    True
    >>> convert_to_boolean('True')
    True
    >>> convert_to_boolean('true')
    True
    >>> convert_to_boolean('1')
    True
    >>> convert_to_boolean(1)
    True
    >>> convert_to_boolean('False')
    False
    """
    truthy = {'t', 'T', 'true', 'True', 'TRUE', '1', 1, True}
    falsy = {'f', 'F', 'false', 'False', 'FALSE', '0', 0, 0.0, False}
    if value is None:
        return None
    elif value in truthy:
        return True
    elif value in falsy:
        return False

    return bool(value)


def get_time_india():
    tz = pytz.timezone('Asia/Kolkata')
    current_time = datetime.now(tz)
    return current_time


def localize_time(t):
    """
    >>> x = datetime(2017, 5, 16)
    >>> localize_time(x)
    datetime.datetime(2017, 5, 16, 0, 0, tzinfo=<DstTzInfo 'Asia/Kolkata' IST+5:30:00 STD>)  # noqa
    """
    tz = pytz.timezone('Asia/Kolkata')
    result = tz.localize(t)
    return result


def convert_string_to_datetime(date, string_format='%Y-%m-%d'):
    """
    >>> convert_string_to_datetime('2017-05-16')
    '2017-05-16T00:00:00'
    """
    return datetime.strptime(date, string_format).isoformat()


def convert_datetime_to_string(date):
    """
    >>> x = datetime(2017, 5, 16)
    >>> convert_datetime_to_string(x)
    '2017-05-16'
    """
    return date.strftime('%Y-%m-%d')


def pre_load_date_fields(in_data, date_fields, date_format='%m-%d-%Y'):
    """
    Converts the date fields into datetime objects

    Args:
        in_data: dict
        date_fields: list<str>
        date_format: str

    Returns: dict

    """
    for date_field in date_fields:
        if date_field in in_data:
            if in_data[date_field] in ['-', None]:
                in_data[date_field] = None
            else:
                in_data[date_field] = convert_string_to_datetime(
                    date=in_data[date_field],
                    string_format=date_format,
                )
    return in_data


if __name__ == '__main__':
    from doctest import testmod
    testmod()



from datetime import datetime
from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
)

from ...database import Base


class OrangeBookPatentModel(Base):
    __tablename__ = 'orange_book_patents'

    id = Column(Integer, primary_key=True)
    appl_no = Column(String(128), index=True)
    appl_type = Column(String(128), nullable=False, index=True)
    delist_flag = Column(String(128))
    drug_product_flag = Column(String(128))
    drug_substance_flag = Column(String(128))
    patent_no = Column(String(128), nullable=False, index=True)
    patent_use_code = Column(String(128), nullable=False)
    product_no = Column(String(128), nullable=False)
    patent_expire_date = Column(DateTime)
    submission_date = Column(DateTime)
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )



from datetime import datetime
from marshmallow import ValidationError
from pymongo.errors import OperationFailure

from ...database import get_mongo_db_session
from .schemas import (
    DocumentTreeResourceSchema,
    DocumentTreeQueryParamsSchema,
    DocumentTreePatchSchema,
)


schema_resource = DocumentTreeResourceSchema()
schema_params = DocumentTreeQueryParamsSchema()
schema_patch = DocumentTreePatchSchema()


class DBException(OperationFailure):
    pass


class DocumentTreeNotFoundException(Exception):
    def __init__(self, message):
        self.message = message


class DocumentTree:
    @staticmethod
    def create(params):
        """
        Args:
            params: Dict
                file_id: int
                value: Dict

        Returns: Dict
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        Args:
            params: Dict
                file_id

        Returns:

        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        mongo_db_session = get_mongo_db_session()
        query = mongo_db_session.document_trees.find(params)
        response = schema_resource.dump(query, many=True).data
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: Dict
                file_id

        Returns: Dict

        Exception:
            ValidationError
            DocumentTreeNotFoundException
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        mongo_db_session = get_mongo_db_session()
        query = mongo_db_session.document_trees.find_one(params)
        if query is None:
            raise DocumentTreeNotFoundException('File id not found!')
        response = schema_resource.dump(query).data
        return response

    @staticmethod
    def update(params):
        """
        :param
            params: dict
                file_id
                value
        :return:
            newly updated object
        :exception:
            DocumentTreeNotFoundException
            ValidationError
            DBException
        """
        data, errors = schema_patch.load(params)
        if errors:
            raise ValidationError(errors)
        data['updated_at'] = datetime.utcnow()
        mongo_db_session = get_mongo_db_session()
        result = mongo_db_session.document_trees.find_one_and_update(
            filter={'file_id': data['file_id']},
            update={'$set': data},
            return_document=True,
        )
        if result is None:
            raise DocumentTreeNotFoundException('File id not found!')
        response = schema_resource.dump(result).data
        return response

    @staticmethod
    def delete(file_id):
        mongo_db_session = get_mongo_db_session()
        result = mongo_db_session.document_trees.find_one_and_delete(
            filter={'file_id': file_id},
        )
        response = schema_resource.dump(result).data
        return response

    @staticmethod
    def update_or_create(params):
        """
        :param
            params: dict
                file_id
                value
        :return:
            newly updated/created object
        :exception:
            ValidationError
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)
        data['updated_at'] = datetime.utcnow()
        mongo_db_session = get_mongo_db_session()
        result = mongo_db_session.document_trees.find_one_and_update(
            filter={'file_id': data['file_id']},
            update={'$set': data},
            return_document=True,
            upsert=True,
        )
        response = schema_resource.dump(result).data
        return response


def _helper_create(data):
    try:
        data['updated_at'] = datetime.utcnow()
        mongo_db_session = get_mongo_db_session()
        mongo_db_session.document_trees.insert_one(data)
        return data['file_id']
    except Exception:
        raise DBException('DB error')



from marshmallow import (
    Schema,
    fields,
    validate,
)


class CompanySECResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    cik_str = fields.String(required=True, validate=not_blank)
    ticker = fields.String(allow_none=True)
    title = fields.String(required=True)
    exchange = fields.String(allow_none=True)
    company_url = fields.String(allow_none=True)
    pipeline_url = fields.String(allow_none=True)
    ir_url = fields.String(allow_none=True)
    is_activated = fields.Boolean(allow_none=True)
    is_biopharma = fields.Boolean(allow_none=True)
    discarded = fields.Boolean(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)


class CompanySECUpdateSchema(Schema):
    id = fields.Integer()
    company_url = fields.String(allow_none=True)
    pipeline_url = fields.String(allow_none=True)
    ir_url = fields.String(allow_none=True)
    is_activated = fields.Boolean(allow_none=True)
    is_biopharma = fields.Boolean(allow_none=True)
    discarded = fields.Boolean(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
)

from cs_models.utils.utils import convert_string_to_datetime


def _pre_load_datetime_fields(in_data):
    date_fields = [
        'document_filing_date',
        'petitioner_grant_date',
        'respondent_grant_date',
    ]

    for date_field in date_fields:
        if date_field in in_data:
            if in_data[date_field] in ['-', None]:
                in_data[date_field] = None
            else:
                in_data[date_field] = convert_string_to_datetime(
                    date=in_data[date_field],
                    string_format='%m-%d-%Y',
                )

    return in_data


class PTAB2DocumentResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    document_category = fields.String()
    document_filing_date = fields.DateTime(allow_none=True)
    document_identifier = fields.String(required=True)
    document_name = fields.String(allow_none=True)
    document_number = fields.String(allow_none=True)
    document_size = fields.String(allow_none=True)
    document_title_text = fields.String(allow_none=True)
    document_type_name = fields.String(allow_none=True)
    filing_party_category = fields.String(allow_none=True)
    media_type_category = fields.String(allow_none=True)
    petitioner_application_number_text = fields.String(allow_none=True)
    petitioner_counsel_name = fields.String(allow_none=True)
    petitioner_grant_date = fields.DateTime(allow_none=True)
    petitioner_group_art_unit_number = fields.String(allow_none=True)
    petitioner_inventor_name = fields.String(allow_none=True)
    petitioner_party_name = fields.String(allow_none=True)
    petitioner_patent_number = fields.String(allow_none=True)
    petitioner_patent_owner_name = fields.String(allow_none=True)
    petitioner_technology_center_number = fields.String(allow_none=True)
    proceeding_number = fields.String(allow_none=True)
    proceeding_type_category = fields.String(allow_none=True)
    respondent_application_number_text = fields.String(allow_none=True)
    respondent_counsel_name = fields.String(allow_none=True)
    respondent_grant_date = fields.DateTime(allow_none=True)
    respondent_group_art_unit_number = fields.String(allow_none=True)
    respondent_inventor_name = fields.String(allow_none=True)
    respondent_party_name = fields.String(allow_none=True)
    respondent_patent_number = fields.String(allow_none=True)
    respondent_patent_owner_name = fields.String(allow_none=True)
    respondent_technology_center_number = fields.String(allow_none=True)
    subproceeding_type_category = fields.String(allow_none=True)
    file_id = fields.Integer(allow_none=True)
    has_smart_doc = fields.Boolean()
    updated_at = fields.DateTime(dump_only=True)

    @pre_load
    def convert_string_to_datetime(self, in_data):
        return _pre_load_datetime_fields(in_data)


class PTAB2DocumentQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    document_identifier = fields.String()
    proceeding_number = fields.String()
    has_smart_doc = fields.Boolean()


class PTAB2DocumentPatchSchema(Schema):
    file_id = fields.Integer()
    has_smart_doc = fields.Boolean()



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    UniqueConstraint,
    Boolean,
)

from ...database import Base


class InterventionModel(Base):
    __tablename__ = "interventions"

    id = Column(Integer, primary_key=True)
    intervention_uid = Column(String(191), nullable=False)
    marketing_category = Column(String(50), nullable=True)
    preferred_term = Column(String(191), nullable=False)
    intervention_type = Column(String(128), nullable=True)
    intervention_url = Column(String(255), nullable=True)
    discontinued = Column(Boolean, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )

    __table_args__ = (UniqueConstraint("intervention_uid"),)



from marshmallow import (
    Schema,
    fields,
    validate,
)


class CrossRefResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    doi = fields.String(required=True, validate=not_blank)
    authors = fields.String(allow_none=True)
    doi_url = fields.String(allow_none=True)
    issn = fields.String(allow_none=True)
    issue = fields.String(allow_none=True)
    journal = fields.String(allow_none=True)
    pages = fields.String(allow_none=True)
    pmid = fields.String(allow_none=True)
    ref_type = fields.String(allow_none=True)
    text = fields.String(allow_none=True)
    title = fields.String(allow_none=True)
    volume = fields.String(allow_none=True)
    year = fields.Integer(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)


class CrossRefQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    doi = fields.String(validate=not_blank)



from marshmallow import (
    Schema,
    fields,
    validate,
)


class CatalystConceptResourceSchema(Schema):
    not_blank = validate.Length(min=1, error="Field cannot be blank")

    id = fields.Integer(dump_only=True)
    concept_name = fields.String(required=True)
    relation = fields.String(required=True)
    property = fields.String(required=True)
    news_id = fields.Integer(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)



from marshmallow import (
    Schema,
    fields,
    validate,
)


class SponsorResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(required=True)
    nct_id = fields.String()
    agency_class = fields.String()
    lead_or_collaborator = fields.String()
    name = fields.String()


class SponsorQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')
    id = fields.Integer()
    name = fields.String()



import json
import copy
from marshmallow import ValidationError
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import (
    NoResultFound,
    MultipleResultsFound,
)
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.mixins.BasicDataMixin import BasicDataMixin
from test.utils import second_equals_first
from src.cs_models.resources.CrossRef import CrossRef
from src.cs_models.resources.PriorArtRefRelation import (
    PriorArtRefRelation,
)


class PriorArtRefRelationResourceTestCase(BasicDataMixin, TestCase):
    def setUp(self):
        super(PriorArtRefRelationResourceTestCase, self).setUp()
        self.inst_cross_ref = CrossRef()
        self.inst = PriorArtRefRelation()

        self.prior_art3 = self.inst_prior_art.create({
            'ptab2_document_id': self.ptab2_document2['id'],
            'tag': 'PA 3',
            'title': 'PA3 detail',
            'exhibit': 'EX 300',
        })

        self.prior_art4 = self.inst_prior_art.create({
            'ptab2_document_id': self.ptab2_document2['id'],
            'tag': 'PA 4',
            'title': 'PA4 detail',
            'exhibit': 'EX 400',
        })

        self.cross_ref1 = self.inst_cross_ref.create({
            'doi': '1000',
            'authors': json.dumps([
                {'family': 'Flechter', 'given': 'Shlomo'},
                {'family': 'Kott', 'given': 'Edna'},
            ]),
            'doi_url': 'http://dx.doi.org/10.1097/00002826-200201000-00002',
            'issn': json.dumps(['0362-5664']),
            'issue': '1',
            'journal': 'Clinical neuropharmacology',
            'pages': '11-5',
            'pmid': '11852290',
            'ref_type': 'journal-article',
            'text': 'Shlomo Flechter et al., Copolymer 1'
                    'Relapsing Forms of Multiple Sclerosis: Open '
                    'Multicenter Study of Alternate-Day Administration.',
            'title': 'Copolymer 1 (glatiramer acetate) in relapsing forms ',
            'volume': '25',
            'year': 2002,
        })

        self.cross_ref2 = self.inst_cross_ref.create({
            'doi': '2000',
            'authors': json.dumps([
                {'family': 'Flechter', 'given': 'Shlomo'},
            ]),
            'doi_url': 'http://dx.doi.org/10.1097/00002826-200201000-00002',
            'issn': json.dumps(['0362-5664']),
            'issue': '1123',
            'journal': 'Clinical neuropharmacology',
            'pages': '11-5',
            'pmid': '11852290',
            'ref_type': 'journal-article',
            'text': 'Shlomo Flechter et al., Copolymer 1'
                    'Relapsing Forms of Multiple Sclerosis: Open Multicenter '
                    'Study of Alternate-Day Administration.',
            'title': 'Copolymer 1 (glatiramer acetate) in relapsing forms ',
            'volume': '25',
            'year': 2002,
        })

        self.pa_ref_relation1 = self.inst.create({
            'prior_art_id': self.prior_art1['id'],
            'cross_ref_id': self.cross_ref1['id'],
        })

        self.pa_ref_relation2 = self.inst.create({
            'prior_art_id': self.prior_art1['id'],
            'patent_id': self.patent1['id'],
        })

        self.pa_ref_relation3 = self.inst.create({
            'prior_art_id': self.prior_art1['id'],
            'patent_application_id': self.patent_application1['id'],
        })

        self.valid_data = {
            'prior_art_id': self.prior_art3['id'],
            'cross_ref_id': self.cross_ref1['id'],
            'patent_id': self.patent1['id'],
            'patent_application_id': self.patent_application1['id'],
        }

    def _get_dict_with_fields(self, base_data, fields_to_keep):
        data = {}
        for key in base_data:
            if key in fields_to_keep:
                data[key] = base_data[key]
        return data

    @parameterized.expand([
        (['prior_art_id'],),
        (['cross_ref_id', 'patent_id', 'patent_application_id'],),
    ])
    def test_create_validation_error_missing_field(self, fields_to_keep):
        base_data = copy.copy(self.valid_data)
        data = self._get_dict_with_fields(base_data, fields_to_keep)

        self.assertRaises(
            ValidationError,
            self.inst.create,
            data,
        )

    def test_create_validation_error_multiple_ref_fields(self):
        data = copy.copy(self.valid_data)
        self.assertRaises(
            ValidationError,
            self.inst.create,
            data,
        )

    @parameterized.expand([
        (['prior_art_id', 'cross_ref_id'],),
        (['prior_art_id', 'patent_id'],),
        (['prior_art_id', 'patent_application_id'],),
    ])
    def test_create_violates_unique_constraint(self, fields_to_keep):
        base_data = {
            'prior_art_id': self.prior_art1['id'],
            'cross_ref_id': self.pa_ref_relation1['cross_ref_id'],
            'patent_id': self.pa_ref_relation2['patent_id'],
            'patent_application_id': (
                self.pa_ref_relation3['patent_application_id']),
        }
        data = self._get_dict_with_fields(base_data, fields_to_keep)

        self.assertRaises(
            IntegrityError,
            self.inst.create,
            data,
        )

    @parameterized.expand([
        ('prior_art_id', ['cross_ref_id']),
        ('cross_ref_id', ['prior_art_id', 'cross_ref_id']),
        ('patent_id', ['prior_art_id', 'patent_id']),
        ('patent_application_id', ['prior_art_id', 'patent_application_id']),
    ])
    def test_create_violates_fk_constraint(
        self,
        field_to_test,
        fields_to_keep,
    ):
        data = self._get_dict_with_fields(self.valid_data, fields_to_keep)
        data[field_to_test] = 999

        self.assertRaises(
            IntegrityError,
            self.inst.create,
            data,
        )

    @parameterized.expand([
        (['prior_art_id', 'cross_ref_id'],),
        (['prior_art_id', 'patent_id'],),
        (['prior_art_id', 'patent_application_id'],),
    ])
    def test_create(self, fields_to_keep):
        base_data = copy.copy(self.valid_data)
        data = self._get_dict_with_fields(base_data, fields_to_keep)
        resp = self.inst.create(data)
        second_equals_first(data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(3, len(resp))

    @parameterized.expand([
        ('id', 'pa_ref_relation1', 'id', 1),
        ('prior_art_id', 'pa_ref_relation1', 'prior_art_id', 3),
        ('cross_ref_id', 'pa_ref_relation1', 'cross_ref_id', 1),
        ('patent_id', 'pa_ref_relation2', 'patent_id', 1),
        (
            'patent_application_id',
            'pa_ref_relation3',
            'patent_application_id',
            1,
        ),
    ])
    def test_read_w_params(
        self,
        field_name,
        attr,
        attr_field,
        expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(3, len(resp))

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    def test_one_raises_no_result_found_exception(self):
        self.assertRaises(
            NoResultFound,
            self.inst.one,
            {'id': 9999},
        )

    def test_one_raises_multiple_results_found_exception(self):
        self.assertRaises(
            MultipleResultsFound,
            self.inst.one,
            {'prior_art_id': self.prior_art1['id']},
        )

    @parameterized.expand([
        ('id', 'pa_ref_relation1'),
        ('cross_ref_id', 'pa_ref_relation1'),
        ('patent_id', 'pa_ref_relation2'),
        ('patent_application_id', 'pa_ref_relation3'),
    ])
    def test_one(self, field_name, attr):
        pa_ref_relation = getattr(self, attr)
        resp = self.inst.one({
            field_name: pa_ref_relation[field_name],
        })
        second_equals_first(
            pa_ref_relation,
            resp,
        )

    def test_delete_not_found(self):
        invalid_id = 99999
        self.assertRaises(
            NoResultFound,
            self.inst.delete,
            invalid_id,
        )

    def test_delete(self):
        response = self.inst.one({'id': self.pa_ref_relation1['id']})
        self.inst.delete(id=response['id'])
        self.assertRaises(
            NoResultFound,
            self.inst.one,
            {'id': self.pa_ref_relation1['id']},
        )



from sqlalchemy.exc import SQLAlchemyError
from datetime import datetime
from marshmallow import ValidationError
from sqlalchemy.orm.exc import NoResultFound

from ...database import db_session
from ...resources.File.models import FileModel
from .schemas import (
    FileResourceSchema,
    FileQueryParamsSchema,
    FilePatchSchema,
)
from ...utils.utils import update_model


schema_resource = FileResourceSchema()
schema_params = FileQueryParamsSchema()
schema_patch = FilePatchSchema()


class FileNotFoundException(Exception):
    def __init__(self, message):
        self.message = message


class File:
    @staticmethod
    def create(params):
        """
        :param
            Dict
                file_format
                s3_bucket_name
                s3_key_name
        :return:
            newly created object
        :exception:
            ValidationError
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        :param
            params: dict
                id
                file_format
                s3_bucket_name
                s3_key_name
        :return:
            queried object
        :exception:
            ValidationError
        """

        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        file_query = _build_query(params=data)
        response = schema_resource.dump(file_query, many=True).data
        return response

    @staticmethod
    def one(params):
        """
        :param
            params: dict
                id
                file_format
                s3_bucket_name
                s3_key_name
        :return:
            queried object
        :exception:
            ValidationError
        """

        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        file_query = _build_query(params=data).one()
        response = schema_resource.dump(file_query).data
        return response

    @staticmethod
    def update(id, params):
        """
        :param
            id: integer: required
            params: dict
                file_format
                s3_bucket_name
                s3_key_name
        :return:
            newly updated object
        :exception:
            FileNotFoundException
            ValidationError
        """
        file_query = db_session.query(FileModel).filter_by(id=id).first()
        if not file_query:
            raise FileNotFoundException('Document not found!')
        data, errors = schema_patch.load(params)
        if errors:
            raise ValidationError(errors)

        response = _helper_update(data, file_query)
        return response

    @staticmethod
    def delete(id):
        """
        :param
            id
        :return:
            delete message
        :exception:
            FileNotFoundException
        """

        file_query = db_session.query(FileModel).filter_by(id=id).first()
        if not file_query:
            raise FileNotFoundException('File does not exist!')
        try:
            db_session.delete(file_query)
            db_session.commit()
            db_session.close()
            return 'Successfully deleted'
        except SQLAlchemyError:
            db_session.rollback()
            db_session.close()
            raise

    @staticmethod
    def create_if_does_not_exist(params):
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        file_query = db_session.query(FileModel).filter_by(
            s3_bucket_name=data.get('s3_bucket_name'),
            s3_key_name=data.get('s3_key_name'),
        ).first()

        if not file_query:
            response = _helper_create(data)
        else:
            response = schema_resource.dump(file_query).data
        return response

    @staticmethod
    def upsert(params):
        """
        Args:
            params: FileResourceSchema

        Returns:
            FileResourceSchema

        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        try:
            query_params = {
                's3_bucket_name': params['s3_bucket_name'],
                's3_key_name': params['s3_key_name'],
            }
            file_query = _build_query(query_params).one()
            response = _helper_update(data, file_query)
        except NoResultFound:
            response = _helper_create(data)
        return response


def _helper_create(data):
    new_file = FileModel(
        file_format=data['file_format'],
        s3_bucket_name=data['s3_bucket_name'],
        s3_key_name=data['s3_key_name'],
        content_length=data.get('content_length'),
        updated_at=datetime.utcnow(),
    )
    try:
        db_session.add(new_file)
        db_session.commit()
        file_query = db_session.query(FileModel).get(new_file.id)
        response = schema_resource.dump(file_query).data
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_update(data, document_query):
    data['id'] = document_query.id
    data['updated_at'] = datetime.utcnow()
    try:
        update_model(data, document_query)
        db_session.commit()
        response = schema_resource.dump(document_query).data
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise


def _build_query(params):
    q = db_session.query(FileModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('file_format'):
        q = q.filter_by(file_format=params.get('file_format'))
    if params.get('s3_bucket_name'):
        q = q.filter_by(s3_bucket_name=params.get('s3_bucket_name'))
    if params.get('s3_key_name'):
        q = q.filter_by(s3_key_name=params.get('s3_key_name'))
    return q



from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm.exc import NoResultFound
from marshmallow import ValidationError

from ...database import db_session
from ...resources.InstitutionProbability.models import (
    InstitutionProbabilityModel,
)
from .schemas import (
    InstitutionProbabilityResourceSchema,
    InstitutionProbabilityQueryParamsSchema,
)
from ...utils.utils import update_model


schema_resource = InstitutionProbabilityResourceSchema()
schema_params = InstitutionProbabilityQueryParamsSchema()


class InstitutionProbability(object):
    @staticmethod
    def create(params):
        """
        Args:
            params: dict(InstitutionProbabilityResourceSchema)

        Returns: InstitutionProbabilityResourceSchema

        Raises:
            ValidationError
            SQLAlchemyError
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        Args:
            params: dict(InstitutionProbabilityQueryParamsSchema)

        Returns: List<InstitutionProbabilityResourceSchema>

        Raises:
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        institution_probability_query = _build_query(params=data)
        response = schema_resource.dump(
            institution_probability_query, many=True).data
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: dict(InstitutionProbabilityQueryParamsSchema)

        Returns: InstitutionProbabilityResourceSchema

        Raises:
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        institution_probability_query = _build_query(params=data).one()
        response = schema_resource.dump(institution_probability_query).data
        return response

    @staticmethod
    def upsert(params):
        """
        Args:
            params: InstitutionProbabilityResourceSchema

        Returns:
            InstitutionProbabilityResourceSchema

        Raises:
            ValidationError
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        try:
            query_params = {
                'ptab_trial_num': params['ptab_trial_num'],
            }
            institution_probability_query = _build_query(query_params).one()
            response = _helper_update(data, institution_probability_query)
        except NoResultFound:
            response = _helper_create(data)
        return response


def _helper_create(data):
    new_institution_probability = InstitutionProbabilityModel(**data)
    try:
        db_session.add(new_institution_probability)
        db_session.commit()
        institution_probability_query = db_session.query(
            InstitutionProbabilityModel).get(
            new_institution_probability.id)
        response = schema_resource.dump(institution_probability_query).data
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_update(data, institution_probability_query):
    data['id'] = institution_probability_query.id
    data['ptab_trial_num'] = institution_probability_query.ptab_trial_num
    try:
        update_model(data, institution_probability_query)
        db_session.commit()
        response = schema_resource.dump(institution_probability_query).data
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise


def _build_query(params):
    q = db_session.query(
        InstitutionProbabilityModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('ptab_trial_num'):
        q = q.filter_by(
            ptab_trial_num=params.get('ptab_trial_num'))
    return q



from marshmallow import (
    Schema,
    fields,
    validate,
)


class TrialStatResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    stat_type = fields.String(required=True, validate=not_blank)
    name = fields.String(required=True, validate=not_blank)
    ref_counts_by_type = fields.String()
    id_count = fields.Integer(allow_none=True)
    updated_at = fields.DateTime(dump_only=True)


class TrialStatQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    stat_type = fields.String(validate=not_blank)
    name = fields.String(validate=not_blank)



from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm.exc import NoResultFound
from marshmallow import ValidationError

from ...database import db_session
from ...resources.CompanyOUS.models import (
    CompanyOUSModel,
)
from .schemas import (
    CompanyOUSResourceSchema,
    CompanyOUSQueryParamsSchema,
)
from ...utils.utils import update_model


schema_resource = CompanyOUSResourceSchema()
schema_params = CompanyOUSQueryParamsSchema()


class CompanyOUS(object):
    @staticmethod
    def create(params):
        """
        Args:
            params: dict(CompanyOUSResourceSchema)

        Returns: CompanyOUSResourceSchema

        Raises:
            ValidationError
            SQLAlchemyError
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)
        response = _helper_create(data)
        return response

    @staticmethod
    def read(params):
        """
        Args:
            params: dict(CompanyOUSQueryParamsSchema)

        Returns: List<CompanyOUSResourceSchema>

        Raises:
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        company_ous_query = _build_query(params=data)
        response = schema_resource.dump(
            company_ous_query, many=True).data
        return response

    @staticmethod
    def one(params):
        """
        Args:
            params: dict(CompanyOUSQueryParamsSchema)

        Returns: CompanyOUSResourceSchema

        Raises:
            ValidationError
        """
        data, errors = schema_params.load(params)
        if errors:
            raise ValidationError(errors)
        company_ous_query = _build_query(params=data).one()
        response = schema_resource.dump(company_ous_query).data
        return response

    @staticmethod
    def upsert(params):
        """
        Args:
            params: CompanyOUSResourceSchema

        Returns:
            CompanyOUSResourceSchema

        Raises:
            ValidationError
        """
        data, errors = schema_resource.load(params)
        if errors:
            raise ValidationError(errors)

        try:
            query_params = {
                'name': params['name'],
            }
            company_ous_query = _build_query(query_params).one()
            response = _helper_update(data, company_ous_query)
        except NoResultFound:
            response = _helper_create(data)
        return response


def _helper_create(data):
    new_company_ous = CompanyOUSModel(**data)
    try:
        db_session.add(new_company_ous)
        db_session.commit()
        company_ous_query = db_session.query(CompanyOUSModel).get(
            new_company_ous.id
        )
        response = schema_resource.dump(company_ous_query).data
        db_session.close()
        return response
    except SQLAlchemyError:
        db_session.rollback()
        db_session.close()
        raise


def _helper_update(data, company_ous_query):
    data['id'] = company_ous_query.id
    data['name'] = company_ous_query.name
    try:
        update_model(data, company_ous_query)
        db_session.commit()
        response = schema_resource.dump(company_ous_query).data
        return response
    except SQLAlchemyError:
        db_session.rollback()
        raise


def _build_query(params):
    q = db_session.query(
        CompanyOUSModel)
    if params.get('id'):
        q = q.filter_by(id=params.get('id'))
    if params.get('name'):
        q = q.filter_by(
            name=params.get('name'))
    if params.get('ticker'):
        q = q.filter_by(ticker=params.get('ticker'))
    return q



from src.cs_models.resources.Patent import Patent
from src.cs_models.resources.AbstractText import AbstractText
from test.mixins.BasicDataMixin import BasicDataMixin
from test.backendtestcase import TestCase
from test.utils import second_equals_first


class AbstractTextResourceTestCase(BasicDataMixin, TestCase):
    def setUp(self):
        super(AbstractTextResourceTestCase, self).setUp()
        self.inst = AbstractText()
        self.inst_patent = Patent()
        self.patent3 = self.inst_patent.create({
            'patent_number': '3000',
            'app_grp_art_number': '1999',
            'jurisdiction': 'USPAT',
        })
        self.abstract_text1 = self.inst.create({
            'patent_id': self.patent1['id'],
            'abstract_text': 'some text1',
        })
        self.abstract_text2 = self.inst.create({
            'patent_id': self.patent2['id'],
            'abstract_text': 'some text2',
        })

    def test_create(self):
        data = {
            'patent_id': self.patent3['id'],
            'abstract_text': 'some text',
        }
        resp = self.inst.create(data)
        second_equals_first(
            {
                'patent_id': self.patent3['id'],
                'abstract_text': data['abstract_text'],
            },
            resp,
        )

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 2)

    def test_read_w_patent_id(self):
        resp = self.inst.read({
            'patent_id': self.patent1['id'],
        })
        self.assertEqual(len(resp), 1)
        second_equals_first(
            self.abstract_text1,
            resp[0],
        )

    def test_update(self):
        new_data = {
            'patent_id': 9999,
            'abstract_text': 'what what',
        }
        resp = self.inst.update(
            id=self.abstract_text1['id'],
            params=new_data,
        )
        second_equals_first(
            {
                'id': self.abstract_text1['id'],
                'patent_id': self.abstract_text1['patent_id'],
                'abstract_text': new_data['abstract_text'],
            },
            resp,
        )

    def test_delete(self):
        response = self.inst.read({'id': self.abstract_text1['id']})
        self.assertEqual(len(response), 1)
        response = self.inst.delete(id=self.abstract_text1['id'])
        self.assertEqual(response, 'Successfully deleted')
        response = self.inst.read({'id': self.abstract_text1['id']})
        self.assertEqual(len(response), 0)



import copy
from marshmallow import ValidationError
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import (
    NoResultFound,
    MultipleResultsFound,
)
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.PatentApplication import PatentApplication


class PatentApplicationResourceTestCase(TestCase):
    def setUp(self):
        super(PatentApplicationResourceTestCase, self).setUp()
        self.inst = PatentApplication()

        self.patent_application1 = self.inst.create({
            'application_number': '1000',
            'jurisdiction': 'USPAT',
        })

        self.patent_application2 = self.inst.create({
            'application_number': '2000',
            'jurisdiction': 'USPAT',
        })

        self.valid_data = {
            'application_number': '3000',
            'jurisdiction': 'USPAT',
        }

    @parameterized.expand([
        ('application_number',),
        ('jurisdiction',),
    ])
    def test_create_validation_error_missing_field(self, field_to_pop):
        base_data = copy.copy(self.valid_data)
        base_data.pop(field_to_pop)
        self.assertRaises(
            ValidationError,
            self.inst.create,
            base_data,
        )

    def test_create_violates_unique_constraint(self):
        self.assertRaises(
            IntegrityError,
            self.inst.create,
            {
                'application_number': (
                    self.patent_application1['application_number']),
                'jurisdiction': self.patent_application1['jurisdiction'],
            },
        )

    def test_create(self):
        data = copy.copy(self.valid_data)
        resp = self.inst.create(data)
        second_equals_first(data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 2)

    @parameterized.expand([
        ('id', 'patent_application1', 'id', 1),
        ('application_number', 'patent_application1', 'application_number', 1),
        ('jurisdiction', 'patent_application1', 'jurisdiction', 2),
    ])
    def test_read_w_params(
        self,
        field_name,
        attr,
        attr_field,
        expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(len(resp), 2)

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    @parameterized.expand([
        ('id', 999, NoResultFound),
        ('jurisdiction', 'USPAT', MultipleResultsFound),
    ])
    def test_one_raises_exception(self, field_name, field_value, exception):
        self.assertRaises(
            exception,
            self.inst.one,
            {
                field_name: field_value,
            },
        )

    @parameterized.expand([
        ('id',),
        ('application_number',),
    ])
    def test_one(self, field_name):
        resp = self.inst.one({
            field_name: self.patent_application1[field_name],
        })
        second_equals_first(
            self.patent_application1,
            resp,
        )

    def test_delete_not_found(self):
        invalid_id = 99999
        self.assertRaises(
            NoResultFound,
            self.inst.delete,
            invalid_id,
        )

    def test_delete(self):
        response = self.inst.one({'id': self.patent_application1['id']})
        self.inst.delete(id=response['id'])
        self.assertRaises(
            NoResultFound,
            self.inst.one,
            {'id': self.patent_application1['id']},
        )

    def test_upsert_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.upsert,
            {
                'application_number': self.valid_data['application_number'],
            }
        )

    def test_upsert_creates_new_entry(self):
        data = copy.copy(self.valid_data)
        self.assertEqual(2, len(self.inst.read({})))
        self.inst.upsert(data)
        self.assertEqual(3, len(self.inst.read({})))

    def test_upsert_updates_existing_row(self):
        data = {
            'application_number': (
                self.patent_application1['application_number']),
            'jurisdiction': self.patent_application1['jurisdiction'],
            'abstract_text': 'Something',
        }
        resp = self.inst.upsert(data)
        second_equals_first(
            data,
            resp,
        )
        self.assertEqual(2, len(self.inst.read({})))



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    UniqueConstraint,
)
from datetime import datetime

from ...database import Base


class PacerCaseModel(Base):
    __tablename__ = 'pacer_cases'

    id = Column(Integer, primary_key=True)
    case_no = Column(String(256))
    court_id = Column(String(191))
    pacer_case_external_id = Column(String(191))
    cause = Column(String(256))
    county = Column(String(256))
    defendant = Column(String(256))
    disposition = Column(String(256))
    filed_date = Column(DateTime)
    flags = Column(String(256))
    jurisdiction = Column(String(256))
    lead_case = Column(String(256))
    nature_of_suit = Column(String(256))
    office = Column(String(256))
    plaintiff = Column(String(256))
    related_case = Column(String(256))
    terminated_date = Column(DateTime)
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )

    __table_args__ = (UniqueConstraint('court_id', 'pacer_case_external_id'),)



from sqlalchemy.orm.exc import (
    NoResultFound,
    MultipleResultsFound,
)
from sqlalchemy.exc import IntegrityError
from parameterized import parameterized
from marshmallow import ValidationError

from src.cs_models.resources.Claim import (
    Claim,
)
from test.mixins.BasicDataMixin import BasicDataMixin
from test.backendtestcase import TestCase
from test.utils import second_equals_first


class ClaimResourceTestCase(BasicDataMixin, TestCase):
    def setUp(self):
        super(ClaimResourceTestCase, self).setUp()
        self.inst = Claim()
        self.claim1 = self.inst.create({
            'patent_id': self.patent1['id'],
            'claim_number': 1,
            'claim_text': 'Some claim 1',
        })
        self.claim2 = self.inst.create({
            'patent_application_id': self.patent_application1['id'],
            'claim_number': 1,
            'claim_text': 'Some claim 2',
        })
        self.claim3 = self.inst.create({
            'patent_application_id': self.patent_application2['id'],
            'claim_number': 2,
            'claim_text': 'Some claim 3',
        })

    def _get_dict_with_fields(self, base_data, fields_to_keep):
        data = {}
        for key in base_data:
            if key in fields_to_keep:
                data[key] = base_data[key]
        return data

    @parameterized.expand([
        (['claim_number', 'claim_text'],),
    ])
    def test_create_validation_error_missing_field(self, fields_to_keep):
        base_data = {
            'patent_id': self.patent1['id'],
            'patent_application_id': self.patent_application1['id'],
            'claim_number': 100,
            'claim_text': 'Some claim 100',
        }
        data = self._get_dict_with_fields(base_data, fields_to_keep)

        self.assertRaises(
            ValidationError,
            self.inst.create,
            data,
        )

    def test_create_validation_error_both_fields(self):
        data = {
            'patent_id': self.patent1['id'],
            'patent_application_id': self.patent_application1['id'],
            'claim_number': 100,
            'claim_text': 'Some claim 100',
        }
        self.assertRaises(
            ValidationError,
            self.inst.create,
            data,
        )

    @parameterized.expand([
        (['claim_number', 'claim_text', 'patent_id'], 'claim1'),
        (['claim_number', 'claim_text', 'patent_application_id'], 'claim2'),
    ])
    def test_create_violates_unique_constraint(
        self,
        fields_to_keep,
        attr_claim,
    ):
        existing_claim = getattr(self, attr_claim)
        base_data = {
            'patent_id': self.patent1['id'],
            'patent_application_id': self.patent_application1['id'],
            'claim_number': existing_claim['claim_number'],
            'claim_text': existing_claim['claim_text'],
        }
        data = self._get_dict_with_fields(base_data, fields_to_keep)

        self.assertRaises(
            IntegrityError,
            self.inst.create,
            data,
        )

    @parameterized.expand([
        ('patent_id', ['claim_number', 'claim_text', 'patent_id']),
        (
            'patent_application_id',
            ['claim_number', 'claim_text', 'patent_application'],
        ),
    ])
    def test_create_violates_fk_constraint(
        self,
        field_to_test,
        fields_to_keep,
    ):
        base_data = {
            'patent_id': self.patent1['id'],
            'patent_application_id': self.patent_application1['id'],
            'claim_number': 123,
            'claim_text': 'Some claim 123',
        }
        data = self._get_dict_with_fields(base_data, fields_to_keep)
        data[field_to_test] = 999

        self.assertRaises(
            IntegrityError,
            self.inst.create,
            data,
        )

    @parameterized.expand([
        (['claim_number', 'claim_text', 'patent_id'],),
        (['claim_number', 'claim_text', 'patent_application_id'],),
    ])
    def test_create(self, fields_to_keep):
        base_data = {
            'patent_id': self.patent1['id'],
            'patent_application_id': self.patent_application1['id'],
            'claim_number': 123,
            'claim_text': 'Some claim 123',
        }
        data = self._get_dict_with_fields(base_data, fields_to_keep)
        resp = self.inst.create(data)
        second_equals_first(data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(3, len(resp))

    @parameterized.expand([
        ('patent_id', 'claim1', 1),
        ('patent_application_id', 'claim2', 1),
        ('claim_number', 'claim3', 1),
    ])
    def test_read_with_params(self, query_field, attr_claim, expected_length):
        resp = self.inst.read({
            query_field: getattr(self, attr_claim)[query_field],
        })
        self.assertEqual(expected_length, len(resp))

    def test_one_raises_no_result_found_exception(self):
        self.assertRaises(
            NoResultFound,
            self.inst.one,
            {'id': 9999},
        )

    def test_one_raises_multiple_results_found_exception(self):
        self.assertRaises(
            MultipleResultsFound,
            self.inst.one,
            {'claim_number': self.claim1['claim_number']},
        )

    @parameterized.expand([
        ('id', 'claim1'),
        ('patent_id', 'claim1'),
        ('patent_application_id', 'claim2'),
        ('claim_number', 'claim3'),
    ])
    def test_one(self, field_name, attr):
        related_matter = getattr(self, attr)
        resp = self.inst.one({
            field_name: related_matter[field_name],
        })
        second_equals_first(
            related_matter,
            resp,
        )

    def test_delete_not_found(self):
        invalid_id = 99999
        self.assertRaises(
            NoResultFound,
            self.inst.delete,
            invalid_id,
        )

    def test_delete(self):
        response = self.inst.one({'id': self.claim1['id']})
        self.inst.delete(id=response['id'])
        self.assertRaises(
            NoResultFound,
            self.inst.one,
            {'id': self.claim1['id']},
        )



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
    UniqueConstraint,
)

from ...database import Base


class PatentCompoundModel(Base):
    __tablename__ = 'patent_compounds'

    id = Column(Integer, primary_key=True)
    patent_id = Column(
        Integer,
        ForeignKey('patents.id'),
        nullable=False,
    )
    cid = Column(String(128), nullable=False)
    compound_name = Column(String(256), nullable=False)
    iupac_name = Column(String(256), nullable=False)
    molecular_formula = Column(String(256), nullable=False)
    molecular_weight = Column(String(256), nullable=False)
    structure_file_id = Column(
        Integer,
        ForeignKey('files.id'),
        nullable=False,
    )

    updated_at = Column(DateTime)

    __table_args__ = (UniqueConstraint('patent_id', 'cid'),)



from marshmallow import (
    Schema,
    fields,
    validate,
)


class DrugIndicationResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    set_id = fields.String(validate=not_blank, required=True)
    lowercase_indication = fields.String(allow_none=True)
    indication = fields.String(allow_none=True)
    label_file_id = fields.Integer(allow_none=True)
    updated_at = fields.DateTime()


class DrugIndicationQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    set_id = fields.String(validate=not_blank)
    lowercase_indication = fields.String(allow_none=True)
    indication = fields.String(allow_none=True)
    label_file_id = fields.Integer(allow_none=True)


class DrugIndicationPatchSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    set_id = fields.String(validate=not_blank)
    lowercase_indication = fields.String(allow_none=True)
    indication = fields.String(allow_none=True)
    label_file_id = fields.Integer(allow_none=True)



from sqlalchemy import (
    Column,
    Integer,
    String,
    Text,
    DateTime,
    UniqueConstraint, Boolean, Float)
from datetime import datetime

from ...database import Base


class InstitutionProbabilityModel(Base):
    __tablename__ = 'institution_probabilities'

    id = Column(Integer, primary_key=True)

    ptab_trial_num = Column(String(190), unique=True, nullable=False)
    is_instituted = Column(Boolean)
    probability_of_institution = Column(Float)
    probability_end_before_trial_end = Column(Float)
    probability_of_all_claims_instituted = Column(Float)
    probability_of_some_claims_instituted = Column(Float)

    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )



import copy
from marshmallow import ValidationError
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import (
    NoResultFound,
)
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.InstitutionProbability import (
    InstitutionProbability,
)


class InstitutionProbabilityResourceTestCase(TestCase):
    def setUp(self):
        super(InstitutionProbabilityResourceTestCase, self).setUp()
        self.inst = InstitutionProbability()

        self.ip1 = self.inst.create({
            'ptab_trial_num': 'IPR101',
            'is_instituted': True,
            'probability_of_institution': 0.345,
            'probability_end_before_trial_end': 0.42,
            'probability_of_all_claims_instituted': 0.345,
            'probability_of_some_claims_instituted': 0.65,
        })

        self.ip2 = self.inst.create({
            'ptab_trial_num': 'IPR201',
            'is_instituted': False,
            'probability_of_institution': 0.43,
            'probability_end_before_trial_end': 0.41,
            'probability_of_all_claims_instituted': 0.34,
            'probability_of_some_claims_instituted': 0.74,
        })

        self.valid_data = {
            'ptab_trial_num': 'IPR301',
            'is_instituted': True,
            'probability_of_institution': 0.45,
            'probability_end_before_trial_end': 0.39,
            'probability_of_all_claims_instituted': 0.4,
            'probability_of_some_claims_instituted': 0.86,
        }

    @parameterized.expand([
        ('ptab_trial_num',),
    ])
    def test_create_validation_error_missing_field(self, field_to_pop):
        base_data = copy.copy(self.valid_data)
        base_data.pop(field_to_pop)
        self.assertRaises(
            ValidationError,
            self.inst.create,
            base_data,
        )

    def test_create_violates_unique_constraint(self):
        self.assertRaises(
            IntegrityError,
            self.inst.create,
            {
                **self.valid_data,
                **{'ptab_trial_num': self.ip1['ptab_trial_num']},
            }
        )

    def test_create(self):
        resp = self.inst.create(self.valid_data)
        second_equals_first(self.valid_data, resp)

    def test_read(self):
        resp = self.inst.read({})
        self.assertEqual(2, len(resp))

    @parameterized.expand([
        ('id', 'ip1', 'id', 1),
        ('ptab_trial_num', 'ip1', 'ptab_trial_num', 1),
    ])
    def test_read_w_params(
        self,
        field_name,
        attr,
        attr_field,
        expected_length,
    ):
        resp = self.inst.read({})
        self.assertEqual(2, len(resp))

        resp = self.inst.read({
            field_name: getattr(self, attr)[attr_field],
        })
        self.assertEqual(expected_length, len(resp))

    @parameterized.expand([
        ('id', 999, NoResultFound),
    ])
    def test_one_raises_exception(self, field_name, field_value, exception):
        self.assertRaises(
            exception,
            self.inst.one,
            {
                field_name: field_value,
            },
        )

    @parameterized.expand([
        ('id',),
        ('ptab_trial_num',),
    ])
    def test_one(self, field_name):
        resp = self.inst.one({
            field_name: self.ip1[field_name],
        })
        second_equals_first(
            self.ip1,
            resp,
        )

    def test_upsert_validation_error(self):
        self.assertRaises(
            ValidationError,
            self.inst.upsert,
            {
                'is_instituted': False,
            }
        )

    def test_upsert_creates_new_entry(self):
        data = copy.copy(self.valid_data)
        self.assertEqual(2, len(self.inst.read({})))
        self.inst.upsert(data)
        self.assertEqual(3, len(self.inst.read({})))

    def test_upsert_updates_existing_row(self):
        data = {
            **self.valid_data,
            **{
                'ptab_trial_num': self.ip1['ptab_trial_num'],
                'is_instituted': not self.ip1['is_instituted'],
            },
        }
        self.assertEqual(2, len(self.inst.read({})))
        resp = self.inst.upsert(data)
        second_equals_first(
            data,
            resp,
        )
        self.assertEqual(2, len(self.inst.read({})))



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    UniqueConstraint,
    ForeignKey,
)
from datetime import datetime

from ...database import Base


class SubsidiarySponsorMappingModel(Base):
    __tablename__ = 'subsidiary_sponsor_mappings'

    id = Column(Integer, primary_key=True)
    sponsor_id = Column(Integer, nullable=False)
    sponsor_name = Column(String(128), nullable=False)
    lead_or_collaborator = Column(String(50))
    subsidiary_id = Column(
        Integer,
        ForeignKey('subsidiaries.id'),
        nullable=False,
    )
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )

    __table_args__ = (UniqueConstraint('sponsor_id', 'subsidiary_id'),)



# pylint: disable=duplicate-code
from test.backendtestcase import TestCase

from sqlalchemy.orm import Session
from sqlalchemy.exc import IntegrityError
from freezegun import freeze_time

from src.cs_models.database import engine, operations
from src.cs_models.resources.File import models, schemas


@freeze_time("2020-01-01")
class OperationsTestCase(TestCase):
    def test_create_inside_context_manager(self):
        """Provides example of how to use the `create` method
        inside a context manager."""
        with operations.session_scope() as session:
            instance = operations.create(
                session=session,
                model=models.FileModel,
                schema=schemas.FileResourceSchema,
                obj={
                    "file_format": "txt",
                    "s3_bucket_name": "test-bucket",
                    "s3_key_name": "key1",
                },
            )
            self.assertEqual(
                schemas.FileResourceSchema().dump(instance).data,
                {
                    "content_length": None,
                    "file_format": "txt",
                    "id": 1,
                    "s3_bucket_name": "test-bucket",
                    "s3_key_name": "key1",
                    "updated_at": "2020-01-01T00:00:00+00:00",
                },
            )

    def test_create(self):
        """Provides an example on how to use the `create` method
        by manually managing the database session."""
        session = Session(bind=engine)
        try:
            instance = operations.create(
                session=session,
                model=models.FileModel,
                schema=schemas.FileResourceSchema,
                obj={
                    "file_format": "txt",
                    "s3_bucket_name": "test-bucket",
                    "s3_key_name": "key1",
                },
            )
            self.assertEqual(
                schemas.FileResourceSchema().dump(instance).data,
                {
                    "content_length": None,
                    "file_format": "txt",
                    "id": 1,
                    "s3_bucket_name": "test-bucket",
                    "s3_key_name": "key1",
                    "updated_at": "2020-01-01T00:00:00+00:00",
                },
            )

            # This commit is not really needed since
            # we commit in `create` but is done for consistency.
            session.commit()
        except Exception:
            session.rollback()
            raise
        finally:
            session.close()

    def test_get_or_create(self):
        """Verifies the if the `obj` doesn't exist, a new row
        is created. If `obj` already exists, then the existing
        row is returned."""
        with operations.session_scope() as session:
            instance, created = operations.get_or_create(
                session=session,
                model=models.FileModel,
                schema=schemas.FileResourceSchema,
                obj={
                    "file_format": "txt",
                    "s3_bucket_name": "test-bucket",
                    "s3_key_name": "key1",
                },
            )
            self.assertEqual(
                schemas.FileResourceSchema().dump(instance).data,
                {
                    "content_length": None,
                    "file_format": "txt",
                    "id": 1,
                    "s3_bucket_name": "test-bucket",
                    "s3_key_name": "key1",
                    "updated_at": "2020-01-01T00:00:00+00:00",
                },
            )
            self.assertTrue(created)

            instance, created = operations.get_or_create(
                session=session,
                model=models.FileModel,
                schema=schemas.FileResourceSchema,
                obj={
                    "file_format": "txt",
                    "s3_bucket_name": "test-bucket",
                    "s3_key_name": "key1",
                },
            )
            self.assertEqual(
                schemas.FileResourceSchema().dump(instance).data,
                {
                    "content_length": None,
                    "file_format": "txt",
                    "id": 1,
                    "s3_bucket_name": "test-bucket",
                    "s3_key_name": "key1",
                    "updated_at": "2020-01-01T00:00:00+00:00",
                },
            )
            self.assertFalse(created)

    def test_update_or_create(self):
        with operations.session_scope() as session:
            instance, created = operations.update_or_create(
                session=session,
                model=models.FileModel,
                schema=schemas.FileResourceSchema,
                query_fields=["s3_key_name", "s3_bucket_name"],
                obj={
                    "file_format": "txt",
                    "s3_bucket_name": "test-bucket",
                    "s3_key_name": "key1",
                },
            )
            self.assertEqual(
                schemas.FileResourceSchema().dump(instance).data,
                {
                    "content_length": None,
                    "file_format": "txt",
                    "id": 1,
                    "s3_bucket_name": "test-bucket",
                    "s3_key_name": "key1",
                    "updated_at": "2020-01-01T00:00:00+00:00",
                },
            )
            self.assertTrue(created)

            instance, created = operations.update_or_create(
                session=session,
                model=models.FileModel,
                schema=schemas.FileResourceSchema,
                query_fields=["s3_key_name", "s3_bucket_name"],
                obj={
                    "file_format": "txt",
                    "s3_bucket_name": "test-bucket",
                    "s3_key_name": "key1",
                },
            )
            self.assertEqual(
                schemas.FileResourceSchema().dump(instance).data,
                {
                    "content_length": None,
                    "file_format": "txt",
                    "id": 1,
                    "s3_bucket_name": "test-bucket",
                    "s3_key_name": "key1",
                    "updated_at": "2020-01-01T00:00:00+00:00",
                },
            )
            self.assertFalse(created)

    def test_bulk_save(self):
        """Verifies that new objects are created in bulk."""
        with operations.session_scope() as session:
            operations.bulk_save(
                session=session,
                instances=[
                    models.FileModel(**obj)
                    for obj in [
                        {
                            "file_format": "txt",
                            "s3_bucket_name": "test-bucket",
                            "s3_key_name": "key1",
                        },
                        {
                            "file_format": "txt",
                            "s3_bucket_name": "test-bucket",
                            "s3_key_name": "key2",
                        },
                    ]
                ],
            )
            self.assertEqual(
                schemas.FileResourceSchema()
                .dump(session.query(models.FileModel).all(), many=True)
                .data,
                [
                    {
                        "content_length": None,
                        "file_format": "txt",
                        "id": 1,
                        "s3_bucket_name": "test-bucket",
                        "s3_key_name": "key1",
                        "updated_at": "2020-01-01T00:00:00+00:00",
                    },
                    {
                        "content_length": None,
                        "file_format": "txt",
                        "id": 2,
                        "s3_bucket_name": "test-bucket",
                        "s3_key_name": "key2",
                        "updated_at": "2020-01-01T00:00:00+00:00",
                    },
                ],
            )

    def test_bulk_save_raises_integrity_error(self):
        with self.assertRaises(IntegrityError):
            with operations.session_scope() as session:
                operations.bulk_save(
                    session=session,
                    instances=[
                        models.FileModel(**obj)
                        for obj in [
                            {
                                "file_format": "txt",
                                "s3_bucket_name": "test-bucket",
                                "s3_key_name": "key1",
                            },
                            {
                                "file_format": "txt",
                                "s3_bucket_name": "test-bucket",
                                "s3_key_name": "key1",
                            },
                        ]
                    ],
                )

    def test_bulk_save_updates_and_creates(self):
        with operations.session_scope() as session:
            instance1 = operations.create(
                session=session,
                model=models.FileModel,
                schema=schemas.FileResourceSchema,
                obj={
                    "file_format": "txt",
                    "s3_bucket_name": "test-bucket",
                    "s3_key_name": "key1",
                },
            )
            instance2 = operations.create(
                session=session,
                model=models.FileModel,
                schema=schemas.FileResourceSchema,
                obj={
                    "file_format": "txt",
                    "s3_bucket_name": "test-bucket",
                    "s3_key_name": "key2",
                },
            )
            instance1.s3_key_name = "key1000"
            instance2.s3_key_name = "key2000"

            operations.bulk_save(
                session=session,
                instances=[
                    instance1,
                    instance2,
                    models.FileModel(
                        **{
                            "file_format": "txt",
                            "s3_bucket_name": "test-bucket",
                            "s3_key_name": "key2",
                        }
                    ),
                    models.FileModel(
                        **{
                            "file_format": "txt",
                            "s3_bucket_name": "test-bucket",
                            "s3_key_name": "key3",
                        }
                    ),
                ],
            )

            self.assertEqual(
                [
                    {
                        "content_length": None,
                        "file_format": "txt",
                        "id": 1,
                        "s3_bucket_name": "test-bucket",
                        "s3_key_name": "key1000",
                        "updated_at": "2020-01-01T00:00:00+00:00",
                    },
                    {
                        "content_length": None,
                        "file_format": "txt",
                        "id": 2,
                        "s3_bucket_name": "test-bucket",
                        "s3_key_name": "key2000",
                        "updated_at": "2020-01-01T00:00:00+00:00",
                    },
                    {
                        "content_length": None,
                        "file_format": "txt",
                        "id": 3,
                        "s3_bucket_name": "test-bucket",
                        "s3_key_name": "key2",
                        "updated_at": "2020-01-01T00:00:00+00:00",
                    },
                    {
                        "content_length": None,
                        "file_format": "txt",
                        "id": 4,
                        "s3_bucket_name": "test-bucket",
                        "s3_key_name": "key3",
                        "updated_at": "2020-01-01T00:00:00+00:00",
                    },
                ],
                schemas.FileResourceSchema()
                .dump(session.query(models.FileModel).all(), many=True)
                .data,
            )

    @staticmethod
    def test_assert_sql_queries():
        """Provides an example on how to assert evaluated SQL queries."""
        with operations.assert_sql(
            [
                (
                    "INSERT INTO files (file_format, s3_bucket_name, s3_key_name, updated_at) "
                    "VALUES (?, ?, ?, ?)",
                    (
                        (
                            "txt",
                            "test-bucket",
                            "key1",
                            "2020-01-01 00:00:00.000000",
                        ),
                        (
                            "txt",
                            "test-bucket",
                            "key2",
                            "2020-01-01 00:00:00.000000",
                        ),
                    ),
                )
            ]
        ):
            with operations.session_scope() as session:
                operations.bulk_save(
                    session=session,
                    instances=[
                        models.FileModel(**obj)
                        for obj in [
                            {
                                "file_format": "txt",
                                "s3_bucket_name": "test-bucket",
                                "s3_key_name": "key1",
                            },
                            {
                                "file_format": "txt",
                                "s3_bucket_name": "test-bucket",
                                "s3_key_name": "key2",
                            },
                        ]
                    ],
                )



from sqlalchemy import (
    Column,
    Integer,
    DateTime,
    ForeignKey,
    UniqueConstraint,
)
from datetime import datetime

from ...database import Base


class RelatedMatterModel(Base):
    __tablename__ = 'related_matters'

    id = Column(Integer, primary_key=True)
    ptab2_document_id = Column(
        Integer,
        ForeignKey('ptab2_documents.id'),
        nullable=False,
    )
    related_pacer_case_id = Column(
        Integer,
        ForeignKey('pacer_cases.id'),
    )
    related_ptab2_proceeding_id = Column(
        Integer,
        ForeignKey('ptab2_proceedings.id'),
    )
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )

    __table_args__ = (
        UniqueConstraint('ptab2_document_id', 'related_pacer_case_id'),
        UniqueConstraint('ptab2_document_id', 'related_ptab2_proceeding_id'),
    )



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
)
from datetime import datetime

from ...database import Base


class SubsidiaryModel(Base):
    __tablename__ = 'subsidiaries'

    id = Column(Integer, primary_key=True)

    name = Column(String(190), unique=True, nullable=False)
    company_sec_id = Column(
        Integer,
        ForeignKey('companies_sec.id'),
        nullable=True,
    )
    company_ous_id = Column(
        Integer,
        ForeignKey('companies_ous.id'),
        nullable=True,
    )
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()


INSTALL_REQUIRES = [
    "marshmallow==2.10.5",
    "PyMySQL==0.9.3",
    "python-dateutil==2.8.0",
    "pytz==2019.1",
    "SQLAlchemy==1.2.15",
    "boto3==1.14.0",
    "botocore==1.17.63",
    "s3transfer==0.3.4",
    "six==1.15.0",
    "urllib3==1.24.2",
    "dnspython==1.16.0",
    "pymongo==3.8.0",
    "psycopg2-binary==2.8.4",
]

if __name__ == "__main__":
    setuptools.setup(
        name="cs-models",
        version="0.0.281",
        author="Harsh Verma",
        author_email="harsh@capitolscience.com",
        description="MySQL db models",
        # long_description=long_description,
        # long_description_content_type='text/markdown',
        url="https://github.com/ezpz76/cs-models",
        packages=setuptools.find_packages(where="src"),
        package_dir={"": "src"},
        classifiers=["Programming Language :: Python :: 3",],
        install_requires=INSTALL_REQUIRES,
        python_requires="~=3.6",
    )



from marshmallow import (
    Schema,
    fields,
    validate,
    pre_load,
    ValidationError,
)
from ...utils.utils import pre_load_date_fields


class DealResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    announcement_date = fields.DateTime(required=True)
    deal_value = fields.Decimal()
    round = fields.String(allow_none=True)
    currency = fields.String(allow_none=True)
    company_sec_id = fields.Integer(allow_none=True)
    company_ous_id = fields.Integer(allow_none=True)
    news_id = fields.Integer(allow_none=True)
    updated_at = fields.DateTime()

    @pre_load
    def check_company_ids(self, in_data):
        if self._get_number_of_company_fields(in_data) != 1:
            raise ValidationError('Provide either company_sec_id or '
                                  'company_ous_id, not both')

    def _get_number_of_company_fields(self, in_data):
        result = 0
        if 'company_sec_id' in in_data:
            if in_data['company_sec_id'] is not None:
                result += 1
        if 'company_ous_id' in in_data:
            if in_data['company_ous_id'] is not None:
                result += 1
        return result

    def convert_string_to_datetime(self, in_data):
        date_fields = ['announcement_date']

        in_data = pre_load_date_fields(
            in_data,
            date_fields,
            date_format='%Y%m%dT%H%M%S',
        )
        return in_data



# pylint: disable=unnecessary-lambda
from datetime import datetime
from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    UniqueConstraint,
)

from ...database import Base


class FileModel(Base):
    __tablename__ = "files"

    id = Column(Integer, primary_key=True)
    file_format = Column(String(128), nullable=False)
    s3_bucket_name = Column(String(128), nullable=False)
    s3_key_name = Column(String(128), nullable=False)
    content_length = Column(Integer)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )

    __table_args__ = (UniqueConstraint("s3_bucket_name", "s3_key_name",),)



from marshmallow import (
    Schema,
    fields,
    validate,
)


class SubsidiarySponsorMappingResourceSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer(dump_only=True)
    sponsor_id = fields.Integer(required=True)
    sponsor_name = fields.String(required=True, validate=not_blank)
    lead_or_collaborator = fields.String()
    subsidiary_id = fields.Integer(required=True)
    updated_at = fields.DateTime(dump_only=True)


class SubsidiarySponsorMappingQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error='Field cannot be blank')

    id = fields.Integer()
    sponsor_id = fields.Integer()
    sponsor_name = fields.String(validate=not_blank)
    lead_or_collaborator = fields.String()
    subsidiary_id = fields.Integer()



from marshmallow import (
    Schema,
    fields,
    validate,
)


class DocumentReferenceResourceSchema(Schema):
    not_blank = validate.Length(min=1, error="Field cannot be blank")

    id = fields.Integer(dump_only=True)
    ptab2_document_id = fields.Integer(required=True)
    type = fields.String(required=True)
    value = fields.String(required=True)

    federal_case_headline = fields.String(allow_none=True)
    federal_case_number = fields.String(allow_none=True)
    link = fields.String(allow_none=True)
    federal_case_file_id = fields.Integer(allow_none=True)
    pages = fields.String(required=True)

    updated_at = fields.DateTime(dump_only=True)


class DocumentReferenceQueryParamsSchema(Schema):
    not_blank = validate.Length(min=1, error="Field cannot be blank")

    id = fields.Integer()
    ptab2_document_id = fields.Integer()
    type = fields.String()
    value = fields.String()
    federal_case_number = fields.String()
    link = fields.String()
    pages = fields.String()


class DocumentReferencePatchSchema(Schema):
    not_blank = validate.Length(min=1, error="Field cannot be blank")

    federal_case_headline = fields.String(allow_none=True)
    federal_case_number = fields.String(allow_none=True)
    link = fields.String(allow_none=True)
    federal_case_file_id = fields.Integer(allow_none=True)
    pages = fields.String(allow_none=False)



import copy

from marshmallow import ValidationError
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm import Query
from parameterized import parameterized

from test.backendtestcase import TestCase
from test.utils import second_equals_first
from src.cs_models.resources.FederalCaseRef import (
    FederalCaseRef,
)
from src.cs_models.resources.FederalCaseRef.models import (
    FederalCaseRefModel,
)
from src.cs_models.resources.File import File
from src.cs_models.database import db_session


class FederalCaseRefResourceTestCase(TestCase):
    def setUp(self):
        super(FederalCaseRefResourceTestCase, self).setUp()

        self.inst_file = File()

        self.inst = FederalCaseRef()

        self.file1 = self.inst_file.create(
            {
                "file_format": "html",
                "s3_bucket_name": "federal-case-pages",
                "s3_key_name": "something.html",
            }
        )

        self.federal_case_ref1 = self.inst.create({
            'case_name': '140 S.Ct. 11',
            'volume': 140,
            'reporter_type': 'S.Ct.',
            'page': 10,
            'title': 'PEARSON v. U.S., Supreme Court of United States.',
            'link': 'https://www.leagle.com/decision/insco20190715c80',
        })

        self.valid_data = {
            'case_name': '947 F.3d 139',
            'volume': 947,
            'reporter_type': 'F.3d',
            'page': 139,
            'title': 'U.S. v. TYSON, United States Court of Appeals, Third Circuit.',
            'link': 'https://www.leagle.com/decision/infco20200115128',
        }

    @parameterized.expand([
        ('case_name',),
        ('volume',),
    ])
    def test_create_validation_error_missing_field(self, field_to_pop):
        base_data = copy.copy(self.valid_data)
        base_data.pop(field_to_pop)
        self.assertRaises(
            ValidationError,
            self.inst.create,
            base_data,
        )

    def test_create(self):
        resp = self.inst.create(self.valid_data)
        second_equals_first(
            self.valid_data,
            resp,
        )

    def test_create_violates_fk_constraint_file_id(self):
        invalid_federal_case_file_id = 12323423
        data = {
            **self.valid_data,
            **{"federal_case_file_id": invalid_federal_case_file_id},
        }
        self.assertRaises(
            IntegrityError, self.inst.create, data,
        )

    def test_delete(self):
        self.inst.delete(id=self.federal_case_ref1['id'])

        q = self._build_query(federal_case_ref_id=self.federal_case_ref1['id'])

        assert q.first() is None

    def test_upsert_creates_new_row(self):
        q = db_session.query(FederalCaseRefModel)
        assert len(q.all()) == 1
        self.inst.upsert(self.valid_data)
        q = db_session.query(FederalCaseRefModel)
        assert len(q.all()) == 2

    def test_upsert_updates_existing_row(self):
        data = {
            'case_name': self.federal_case_ref1['case_name'],
            'volume': self.federal_case_ref1['volume'],
            'reporter_type': self.federal_case_ref1['reporter_type'],
            'page': self.federal_case_ref1['page'],
            'title': self.federal_case_ref1['title'],
            'link': self.federal_case_ref1['link'],
        }
        q = db_session.query(FederalCaseRefModel)
        assert len(q.all()) == 1
        response = self.inst.upsert(data)
        q = db_session.query(FederalCaseRefModel)
        assert len(q.all()) == 1
        second_equals_first(data, response)

    @parameterized.expand([
        ('volume',),
        ('page',),
    ])
    def test_upsert_raises_validation_error(self, field_to_pop):
        base_data = copy.copy(self.valid_data)
        base_data.pop(field_to_pop)
        self.assertRaises(
            ValidationError,
            self.inst.upsert,
            base_data,
        )

    @staticmethod
    def _build_query(federal_case_ref_id: int) -> Query:
        q = db_session.query(
            FederalCaseRefModel,
        ).filter_by(id=federal_case_ref_id)
        return q



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    DateTime,
    String,
    ForeignKey,
    UniqueConstraint,
)

from ...database import Base


class ViewInterventionNormTargetModel(Base):
    __tablename__ = "_view_intervention_norm_targets"

    id = Column(Integer, primary_key=True)
    intervention_norm_cui = Column(String(191), nullable=False, index=True)
    target_id = Column(
        Integer,
        ForeignKey('targets.id'),
        nullable=False,
    )
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )

    __table_args__ = (UniqueConstraint("intervention_norm_cui", "target_id"),)



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    DateTime,
    String,
    Text,
)

from ...database import Base


class InvestorModel(Base):
    __tablename__ = "investors"

    id = Column(Integer, primary_key=True)
    name = Column(String(191), nullable=False, index=True)
    aliases = Column(Text, nullable=True)
    website = Column(String(256), nullable=True)
    type = Column(String(50), nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from datetime import datetime

from sqlalchemy import (
    Column,
    Integer,
    String,
    ForeignKey,
    DateTime,
    Text,
)

from ...database import Base


class CatalystConceptModel(Base):
    __tablename__ = "catalyst_concepts"

    id = Column(Integer, primary_key=True)
    concept_name = Column(String(128), nullable=False, index=True)
    relation = Column(String(128), nullable=False)
    property = Column(Text, nullable=False)
    news_id = Column(
        Integer,
        ForeignKey('newswires.id'),
        nullable=True,
    )
    updated_at = Column(
        DateTime,
        nullable=False,
        # https://stackoverflow.com/questions/58776476/why-doesnt-freezegun-work-with-sqlalchemy-default-values
        default=lambda: datetime.utcnow(),
        onupdate=lambda: datetime.utcnow(),
    )



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    Text,
)
from datetime import datetime

from ...database import Base


class TargetModel(Base):
    __tablename__ = 'targets'

    id = Column(Integer, primary_key=True)

    cui = Column(String(50), unique=True, nullable=False)
    name = Column(String(191), nullable=False)
    symbol = Column(String(50), nullable=False)
    bio_type = Column(String(191), nullable=True)
    pathways = Column(Text, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )



from sqlalchemy import (
    Column,
    Integer,
    ForeignKey,
    DateTime,
    Boolean,
)
from datetime import datetime

from ...database import Base


class TargetPathwayMapModel(Base):
    __tablename__ = 'target_pathway_mapping'

    id = Column(Integer, primary_key=True)
    target_id = Column(
        Integer,
        ForeignKey('targets.id'),
        nullable=False,
    )
    pathway_id = Column(
        Integer,
        ForeignKey('pathways.id'),
        nullable=False,
    )
    is_deleted = Column(Boolean, nullable=True)
    updated_at = Column(
        DateTime,
        nullable=False,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )



import os

import boto3
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import scoped_session, sessionmaker


def _get_ssm_parameters():
    result = {}
    session = boto3.Session(region_name="us-east-1")
    ssm = session.client("ssm")
    ssm_parameters = ssm.get_parameters(
        Names=["mysqlHost", "mysqlUser", "mysqlPassword", "mysqlDB"],
        WithDecryption=True | False,
    )["Parameters"]
    for parameter in ssm_parameters:
        result[parameter["Name"]] = parameter["Value"]
    return result


def _get_credentials():
    result = {
        "mysqlHost": os.getenv("MYSQL_HOST", "127.0.0.1"),
        "mysqlUser": os.getenv("MYSQL_USER", "root"),
        "mysqlPassword": os.getenv("MYSQL_PASSWORD", "testpass"),
        "mysqlDB": os.getenv("MYSQL_DB", "dev_db"),
    }
    if os.getenv("FLASK_CONFIGURATION") == "production":
        result = _get_ssm_parameters()
    return result


def _get_mysql_uri():
    environment = os.getenv("FLASK_CONFIGURATION")
    if environment in ("testing", "ci"):
        return "sqlite://"

    credentials = _get_credentials()
    return "mysql+pymysql://%s:%s@%s/%s?charset=utf8mb4" % (
        credentials["mysqlUser"],
        credentials["mysqlPassword"],
        credentials["mysqlHost"],
        credentials["mysqlDB"],
    )


def import_models():
    # import all modules here that might define models so that
    # they will be registered properly on the metadata.  Otherwise
    # you will have to import them first before calling init_db()
    # this usually happens when a model declares a FK constraint but the
    # referenced table hasn't been created yet
    # import src.cs_models.resources.all_models  # noqa
    pass


mysql_db_uri = _get_mysql_uri()
engine = create_engine(mysql_db_uri, convert_unicode=True)

db_session = scoped_session(
    sessionmaker(autocommit=False, autoflush=False, bind=engine)
)

Base = declarative_base()
Base.query = db_session.query_property()



from sqlalchemy import (
    Column,
    Integer,
    String,
    DateTime,
    ForeignKey,
    UniqueConstraint,
)
from datetime import datetime

from ...database import Base


class CompanySECFilingModel(Base):
    __tablename__ = 'companies_sec_filings'

    id = Column(Integer, primary_key=True)
    company_sec_id = Column(
        Integer,
        ForeignKey('companies_sec.id'),
        nullable=True,
    )
    company_ous_id = Column(
        Integer,
        ForeignKey('companies_ous.id'),
        nullable=True,
    )
    cik_str = Column(String(128), nullable=True)
    sec_form_header = Column(String(50))
    sec_filing_date = Column(DateTime)
    sec_accession_number = Column(String(128))
    sec_index_url = Column(String(128))
    sec_url = Column(String(128))
    document_group = Column(String(50), nullable=True)
    section_name = Column(String(50), nullable=True)
    extraction_method = Column(String(50), nullable=True)
    excerpt_file_id = Column(Integer, ForeignKey("files.id"), nullable=True)
    source_file_id = Column(Integer, ForeignKey("files.id"), nullable=True)
    updated_at = Column(
        DateTime,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
    )

    __table_args__ = (UniqueConstraint('sec_accession_number', 'document_group', 'section_name'),)
